# Copyright 2016-2020 PeppyMeter peppy.player@gmail.com
# 
# This file is part of PeppyMeter.
# 
# PeppyMeter is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# 
# PeppyMeter is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with PeppyMeter. If not, see <http://www.gnu.org/licenses/>.

import time
import os.path
import pygame as pg
import subprocess
import requests
import io

from socketIO_client import SocketIO
from threading import Thread, Timer
from configparser import ConfigParser
from configfileparser import METER, ALBUMART_POS, ALBUMART_DIM, ALBUMBORDER, PLAY_TITLE_POS, PLAY_ARTIST_POS, PLAY_ALBUM_POS, PLAY_MAX, \
  FONTSIZE_LIGHT, FONTSIZE_REGULAR, FONTSIZE_BOLD, FONTCOLOR, PLAY_TITLE_STYLE, PLAY_ARTIST_STYLE, PLAY_ALBUM_STYLE, FONT_STYLE_L, FONT_STYLE_R, FONT_STYLE_B, \
  FONT_PATH, FONT_LIGHT, FONT_REGULAR, FONT_BOLD, TIME_REMAINING_POS, FONTSIZE_DIGI, TIMECOLOR
  
class AlbumartAnimator(Thread):
    """ Provides show albumart in a separate thread """
    
    def __init__(self, util, base):
        """ Initializer

        :param util: utility class
        :param base: complete meter class
        """
        Thread.__init__(self)
        self.screen = util.PYGAME_SCREEN		
        self.base = base
        self.run_flag = True
        self.util = util
        self.meter_config = util.meter_config
        self.meter_name = self.meter_config[METER]
        self.meter_section = self.meter_config[self.meter_name]
        
    def run(self):
        """ Thread method. Converts volume value into the needle angle and displays corresponding sprite. """
 		
        def on_push_state(*args):
            #print(args[0]['status'])
            if args[0]['status'] == 'play':
                info_changed = 0
                #print('on_push_state')
                if args[0]['albumart'] != self.albumart_mem:
                    self.albumart_mem = args[0]['albumart']
                    title_factory.get_albumart_data(self.albumart_mem)
				
                    # clear albumart area
                    if self.meter_section[ALBUMART_POS]:
                        if self.first_run:
                            self.AABackup = self.screen.subsurface(self.meter_section[ALBUMART_POS][0], self.meter_section[ALBUMART_POS][1], self.meter_section[ALBUMART_DIM][0], self.meter_section[ALBUMART_DIM][1]).copy()
                        self.screen.blit(self.AABackup, (self.meter_section[ALBUMART_POS][0], self.meter_section[ALBUMART_POS][1]))
                
                    # draw albumart
                    imgrect = title_factory.aa_img.get_rect()
                    imgrect.left = self.meter_section[ALBUMART_POS][0]
                    imgrect.top = self.meter_section[ALBUMART_POS][1]

                    self.screen.blit(title_factory.aa_img, (imgrect.left, imgrect.top)) # draw albumart
                    if self.meter_section[ALBUMBORDER]:
                        pg.draw.rect(self.screen, title_factory.fontcolor, imgrect, self.meter_section[ALBUMBORDER])   # draw border
                    info_changed += 1

                # draw title info if title changed
                if args[0]['title'] != self.title_mem:
                    self.title_mem = args[0]['title']
                    title_factory.get_title_data(args[0])
                    title_factory.render_text(self.first_run)
                    info_changed += 1

                # draw reamining time
                if self.meter_section[TIME_REMAINING_POS]:
                    duration = args[0]['duration'] if 'duration' in args[0] else 0
                    seek = args[0]['seek'] if 'seek' in args[0] and args[0]['seek'] is not None else 0
                    service = args[0]['service'] if 'service' in args[0] else ''					
                    self.time_args = [duration, seek, service]
				
                    try:
                        self.timer_initial = True
                        timer.start()
                    except:
                        pass
							
                self.first_run = False
                self.status_mem = 'play'

                if info_changed > 0:
                    #print ('Update')
                    self.base.update()

            # simulate mouse event, if pause pressed
            elif args[0]['status'] == 'pause' and self.status_mem == 'play':
                #print ('pause')
                self.status_mem = 'pause'
                pg.event.post(pg.event.Event(pg.MOUSEBUTTONDOWN))

            # simulate mouse event, if stop pressed for webradio
            elif args[0]['service'] == 'webradio' and args[0]['status'] == 'stop' and self.status_mem == 'play':
                #print ('pause')
                self.status_mem = 'stop'
                pg.event.post(pg.event.Event(pg.MOUSEBUTTONDOWN))
				
            else:
                self.status_mem = 'other'
				
        def remaining_time():
            title_factory.get_time_data(self.time_args, self.timer_initial)
            title_factory.render_time(self.first_run_digi)
            self.base.update_rectangle(title_factory.timeRec)
            self.timer_initial = False
            self.first_run_digi = False
			
        def on_connect():
            #print('connect')
            socketIO.on('pushState', on_push_state)
            socketIO.emit('getState', '', on_push_state)

        def on_disconnect():
            #print('disconnect')
            timer.cancel()
			
        self.base.albumart_started.set() # set the running event
        self.albumart_mem = ''
        self.title_mem = ''
        self.status_mem = 'pause'
        self.first_run = True
        self.first_run_digi = True
        timer = RepeatTimer(1, remaining_time)
		
        title_factory = ImageTitleFactory(self.util)
        title_factory.load_fonts() # load fonts for title info

        socketIO = SocketIO('localhost', 3000)
        socketIO.once('connect', on_connect)
        socketIO.on('disconnect', on_disconnect)
		
		# wait while run_flag true 
        while self.run_flag:
            socketIO.wait(1)		
        socketIO.disconnect()
	
		
			
class ImageTitleFactory():
    """ Provides show albumart in a separate thread """
    
    def __init__(self, util):#, titleInfoPath):
        """ Initializer

        :param util: utility class
        """

        self.screen = util.PYGAME_SCREEN		
        self.util = util
        self.meter_config = util.meter_config
        #self.img_path = titleInfoPath
        self.config = ConfigParser()
        self.meter_section = util.meter_config[util.meter_config[METER]]
		
    def load_fonts(self):
        """ load fonts for titleinfo """
        FontPath = self.meter_config[FONT_PATH]
        if os.path.exists(FontPath + self.meter_config[FONT_LIGHT]):
            self.fontL = pg.font.Font(FontPath + self.meter_config[FONT_LIGHT], self.meter_section[FONTSIZE_LIGHT])
        else:
            self.fontL = pg.font.SysFont(None, 50)
        if os.path.exists(FontPath + self.meter_config[FONT_REGULAR]):
            self.fontR = pg.font.Font(FontPath + self.meter_config[FONT_REGULAR], self.meter_section[FONTSIZE_REGULAR])
        else:
            self.fontR = pg.font.SysFont(None, 50)
        if os.path.exists(FontPath + self.meter_config[FONT_BOLD]):		
            self.fontB = pg.font.Font(FontPath + self.meter_config[FONT_BOLD], self.meter_section[FONTSIZE_BOLD])
        else:
            self.fontB = pg.font.SysFont(None, 70, bold=True)

        FontPathDigi = os.path.dirname(os.path.realpath(__file__)) + '/fonts/DSEG7Classic-Italic.ttf'			
        if os.path.exists(FontPathDigi) and self.meter_section[FONTSIZE_DIGI]:
            self.fontDigi = pg.font.Font(FontPathDigi, self.meter_section[FONTSIZE_DIGI])
        else:
            self.fontDigi = pg.font.SysFont(None, 40)
			
        self.fontcolor = self.meter_section[FONTCOLOR]
        #green = (84, 198, 136)
		
    def get_title_data(self, play_info):
        """ get title infos from argument """
			
        self.playinfo_title = play_info['title'] if play_info['title'] is not None else ''
        self.playinfo_artist = play_info['artist'] if play_info['artist'] is not None else ''
        self.playinfo_album = play_info['album'] if play_info['album'] is not None else ''
        if not self.meter_section[PLAY_ALBUM_POS] and self.playinfo_album != '':
            self.playinfo_artist = self.playinfo_artist + " - " + self.playinfo_album

    def get_albumart_data(self, play_info):
        """ get albumart infos from argument """	
                
        albumart = play_info
        if len(albumart) == 0:
            albumart = 'http://localhost:3000/albumart'			
        if 'http' not in albumart:
            albumart = 'http://localhost:3000' + play_info

        #print (albumart)
        response = requests.get(albumart)
        self.aa_img = pg.image.load(io.BytesIO(response.content))
        if self.meter_section[ALBUMART_DIM]:
            self.aa_img = pg.transform.scale(self.aa_img, self.meter_section[ALBUMART_DIM])
					

    def get_time_data(self, time_args, timer_init):
        """ get time data """

        self.NoTime = False
        if timer_init:
            self.seek_new = 0
            self.seek_mem = 32000
		
        seek_current = int(float(time_args[1])/1000)
        if seek_current == self.seek_mem:
            self.seek_new += 1
        else:
            self.seek_new = seek_current
            self.seek_mem = seek_current

        if time_args[2] == 'webradio':
            self.remain = time_args[0] 		
            if time_args[0] == 0:
                self.NoTime = True
        else:
            self.remain = 0 if time_args[0] - self.seek_new <= 0 else time_args[0] - self.seek_new			
        self.timecolor = self.meter_section[TIMECOLOR] if self.remain > 10 else (242,0,0)
        self.remain = '{:02d}:{:02d}'.format( self.remain // 60, self.remain %60)
        #print(self.remain)
		
    def render_time(self, firstrun):
        """ render time info """

        imgDigi = self.fontDigi.render(self.remain, True, self.timecolor)			
        self.timeRec = pg.Rect(self.meter_section[TIME_REMAINING_POS][0], self.meter_section[TIME_REMAINING_POS][1], imgDigi.get_width(), imgDigi.get_height())

        if firstrun:
            self.imgTimeBackup = self.screen.subsurface(self.timeRec).copy()
        self.screen.blit(self.imgTimeBackup, self.meter_section[TIME_REMAINING_POS])
        if self.NoTime == False:
            self.screen.blit(imgDigi, self.meter_section[TIME_REMAINING_POS])
		
    def render_text(self, firstrun):
        """ render of textobjects """

        # title
        imgTitle_long = self.fontB.render(self.playinfo_title, True, self.fontcolor )
        if self.meter_section[PLAY_TITLE_STYLE] == FONT_STYLE_L:
            imgTitle_long = self.fontL.render(self.playinfo_title, True, self.fontcolor )
        elif self.meter_section[PLAY_TITLE_STYLE] == FONT_STYLE_R:
            imgTitle_long = self.fontR.render(self.playinfo_title, True, self.fontcolor )
			
        # artist
        imgArtist_long = self.fontL.render(self.playinfo_artist, True, self.fontcolor )
        if self.meter_section[PLAY_ARTIST_STYLE] == FONT_STYLE_R:
            imgArtist_long = self.fontR.render(self.playinfo_artist, True, self.fontcolor )
        elif self.meter_section[PLAY_ARTIST_STYLE] == FONT_STYLE_B:
            imgArtist_long = self.fontB.render(self.playinfo_artist, True, self.fontcolor )

        # album
        imgAlbum_long = self.fontL.render(self.playinfo_album, True, self.fontcolor )
        if self.meter_section[PLAY_ALBUM_STYLE] == FONT_STYLE_R:
            imgAlbum_long = self.fontR.render(self.playinfo_album, True, self.fontcolor )
        elif self.meter_section[PLAY_ALBUM_STYLE] == FONT_STYLE_B:
            imgAlbum_long = self.fontB.render(self.playinfo_album, True, self.fontcolor )

        # crop the with to max value
        if self.meter_section[PLAY_TITLE_POS] and self.meter_section[PLAY_MAX]:
            if firstrun:
                self.imgTitleBackup = self.screen.subsurface((self.meter_section[PLAY_TITLE_POS][0], self.meter_section[PLAY_TITLE_POS][1], self.meter_section[PLAY_MAX], imgTitle_long.get_height())).copy()
            self.screen.blit(self.imgTitleBackup, self.meter_section[PLAY_TITLE_POS])
            self.screen.blit(imgTitle_long, self.meter_section[PLAY_TITLE_POS], self.imgTitleBackup.get_rect())
			
        if self.meter_section[PLAY_ARTIST_POS] and self.meter_section[PLAY_MAX]:		
            if firstrun:
                self.imgArtistBackup = self.screen.subsurface((self.meter_section[PLAY_ARTIST_POS][0], self.meter_section[PLAY_ARTIST_POS][1], self.meter_section[PLAY_MAX], imgArtist_long.get_height())).copy()
            self.screen.blit(self.imgArtistBackup, self.meter_section[PLAY_ARTIST_POS])
            self.screen.blit(imgArtist_long, self.meter_section[PLAY_ARTIST_POS], self.imgTitleBackup.get_rect())
		
        if self.meter_section[PLAY_ALBUM_POS] and self.meter_section[PLAY_MAX]:
            if firstrun:
                self.imgAlbumBackup = self.screen.subsurface((self.meter_section[PLAY_ALBUM_POS][0], self.meter_section[PLAY_ALBUM_POS][1], self.meter_section[PLAY_MAX], imgAlbum_long.get_height())).copy()
            self.screen.blit(self.imgAlbumBackup, self.meter_section[PLAY_ALBUM_POS])
            self.screen.blit(imgAlbum_long, self.meter_section[PLAY_ALBUM_POS], self.imgAlbumBackup.get_rect())

# RepeatTimer for remaining time
class RepeatTimer(Timer):
    def run(self):
        while not self.finished.wait(self.interval):
            self.function(*self.args, **self.kwargs)
			